@@ -10,9 +10,11 @@
 #  include "pal_apple.h"
 #  include "pal_freebsd.h"
 #  include "pal_freebsd_kernel.h"
+#  include "pal_haiku.h"
 #  include "pal_linux.h"
 #  include "pal_netbsd.h"
 #  include "pal_openbsd.h"
+#  include "pal_solaris.h"
 #  include "pal_windows.h"
 #endif
 #include "pal_plain.h"
@@ -31,10 +33,14 @@ namespace snmalloc
     PALFreeBSDKernel;
 #  elif defined(__FreeBSD__)
     PALFreeBSD;
+#  elif defined(__HAIKU__)
+    PALHaiku;
 #  elif defined(__NetBSD__)
     PALNetBSD;
 #  elif defined(__OpenBSD__)
     PALOpenBSD;
+#  elif defined(__sun)
+    PALSolaris;
 #  else
 #    error Unsupported platform
 #  endif
